/**
 * File: FreeRunningTimer.c
 * Author: Dylan Harootunian
 * Created January 19, 2021 10:20PM
 * ECE121 Winter 2021
 */


#include "xc.h"
#include "sys/attribs.h"
#include "stdio.h"
#include "stdlib.h"
#include "BOARD.h"
#include "Protocol.h"
#include "stdint.h"
#include "MessageIDs.h"
#include "GenericTypeDefs.h"
#include "FreeRunningTimer.h"
#include "FrequencyGenerator.h"
#include "ADCFilter.h"

static char testMessage[MAXPAYLOADLENGTH];
static short index = 0;
static short p =0;
static short A[3][FILTERLENGTH];
static short payload[FILTERLENGTH];
static short weight[FILTERLENGTH];
static short Fil[3][FILTERLENGTH];

union readreturn {
    struct {
        short raw;
        short filt;
    }; unsigned char len[4];
};
    
//int main(void){
//    BOARD_Init();
//    Protocol_Init();
//    sprintf(testMessage, "ADCFilter Test Compiled at %s %s", __DATE__, __TIME__);
//    Protocol_SendDebugMessage(testMessage);
//    FreeRunningTimer_Init();
//    FrequencyGenerator_Init();
//    ADCFilter_Init();
//    unsigned short CurFrequency;
//    unsigned char freqState;
//    int time = 10;
//    while(1){
//        if (Protocol_IsMessageAvailable()) {
//            if (Protocol_ReadNextID() == ID_LAB3_SET_FREQUENCY) {
//                Protocol_GetPayload(&CurFrequency);
//                CurFrequency = Protocol_ShortEndednessConversion(CurFrequency);
//                FrequencyGenerator_SetFrequency(CurFrequency);
//            }
//            if (Protocol_ReadNextID() == ID_LAB3_FREQUENCY_ONOFF) {
//                Protocol_GetPayload(&freqState);
//                if (freqState) {
//                    FrequencyGenerator_On();
//                } else {
//                    FrequencyGenerator_Off();
//                }
//            }
//            if(Protocol_ReadNextID() == ID_ADC_SELECT_CHANNEL){
//                Protocol_GetPayload(&p);
//                Protocol_SendMessage(0, ID_ADC_SELECT_CHANNEL_RESP, 0x00);
//            }
//            
//            if(Protocol_ReadNextID() == ID_ADC_FILTER_VALUES){
//                Protocol_GetPayload(weight);
//                unsigned short i;
//                for(i=0; i<FILTERLENGTH; i++){
//                    weight[i] = Protocol_ShortEndednessConversion(weight[i]);
//                }
////                Protocol_SendMessage(64, ID_ADC_FILTER_VALUES, &weight);
//                ADCFilter_SetWeights(p,weight);
//                Protocol_SendMessage(1, ID_ADC_FILTER_VALUES_RESP, &p);
//            }
//        }
//        if(FreeRunningTimer_GetMilliSeconds()>time){
//            time+=10;
////            short rnf[3];
////            short filtered = ADCFilter_FilteredReading(p);
////            short raw = ADCFilter_RawReading(p);
////            rnf[0]=(raw&0xFF00)>>8;
////            rnf[1]=(raw&0x00FF);
////            rnf[2]=(filtered&0xFF00)>>8;
////            rnf[3]=(filtered&0x00FF);
////            
////            Protocol_SendMessage(4, ID_ADC_READING, &rnf);
//            union readreturn read;
//            read.raw = 0;
//            read.filt = 0;
//            read.raw = Protocol_ShortEndednessConversion(ADCFilter_RawReading(p));
//
//            read.filt = Protocol_ShortEndednessConversion(ADCFilter_FilteredReading(p));
//
//            Protocol_SendMessage(4, ID_ADC_READING, &read.len);
//        }
//    }
//    return SUCCESS;
//}

void __ISR ( _ADC_VECTOR ) ADCIntHandler ( void ) {
//    LEDS_SET(1);
    IFS1bits.AD1IF = 0;//clears ADC interupt flag 
    A[0][index]=ADC1BUF0;
    A[1][index]=ADC1BUF1;
    A[2][index]=ADC1BUF2;
    A[3][index]=ADC1BUF3;
    index=(index+1)%FILTERLENGTH;
    return;
}
/**
 * @Function ADCFilter_Init(void)
 * @param None
 * @return SUCCESS or ERROR
 * @brief initializes ADC system along with naive filters */
int ADCFilter_Init(void){
//    int i;
//    int j;
//    for(i=0; i<4;i++){
//        for(j=0;j<FILTERLENGTH;j++){
//            Fil[i][j]=1;
//        }
//    }
    AD1CON1=0;
    AD1CON2=0;
    AD1CON3=0;
//    IEC1bits.AD1IE = 0;//disables ADC interupt

    //connect the desired pins to the ADC
    AD1PCFGbits.PCFG2 = 0;//a0->an2
    AD1PCFGbits.PCFG4 = 0;//a1->an4
    AD1PCFGbits.PCFG8 = 0;//a2->an8
    AD1PCFGbits.PCFG10 = 0;//a3->an10

    //add those same pins to the scanner
    AD1CSSLbits.CSSL2 = 1;//a0->an2
    AD1CSSLbits.CSSL4 = 1;//a1->an4
    AD1CSSLbits.CSSL8 = 1;//a2->an8
    AD1CSSLbits.CSSL10 = 1;//a3->an10
//    AD1PCFG = 0xFAEB;
//    AD1CSSL = 0x0514;
    
    TRISBbits.TRISB2 =1;
    TRISBbits.TRISB4 =1;
    TRISBbits.TRISB8 =1;
    TRISBbits.TRISB10 =1;
    
    AD1CON1bits.ASAM = 1;//auto sample
    AD1CON1bits.FORM = 0;//interger(unsigned) 16 bit mode
    AD1CON1bits.SSRC = 0b111;//auto convert
    
    AD1CON2bits.VCFG = 0b100;//setting the ADC to use internal voltage references
    AD1CON2bits.CSCNA = 1;//enables scan mode
    AD1CON2bits.SMPI = 0b0011;//interrupts at the completion every 4 samples(4pins) 
    
    AD1CON2bits.BUFM =0;//16 bit buffer
    AD1CON3bits.ADRC = 0;//clock derived from peripheral bus clock
    
    AD1CON3bits.ADCS = 0b10101101;//348Tb
    AD1CON3bits.SAMC = 0b10000;//16Tad
    
    IFS1bits.AD1IF = 0;//clears ADC interupt flag
    IPC6bits.AD1IP = 2;//sets priority
    IEC1bits.AD1IE = 1;//enables ADC interupt
    AD1CON1bits.ON=1;
    
//    LEDS_SET(2);
    return;
}

/**
 * @Function ADCFilter_RawReading(short pin)
 * @param pin, which channel to return
 * @return un-filtered AD Value
 * @brief returns current reading for desired channel */
short ADCFilter_RawReading(short pin){
//    Protocol_SendMessage(64, ID_ADC_FILTER_VALUES, &A[pin]);
     return A[pin][index];
}

/**
 * @Function ADCFilter_FilteredReading(short pin)
 * @param pin, which channel to return
 * @return Filtered AD Value
 * @brief returns filtered signal using weights loaded for that channel */
short ADCFilter_FilteredReading(short pin){
    short start = index;
//    short v[FILTERLENGTH];
//    short f[FILTERLENGTH];
//    int i;
//    for(i=0;i<FILTERLENGTH;i++){
//         v[i] = A[pin][i];
//    }
//    for(i=0;i<FILTERLENGTH;i++){
//         f[i] = Fil[pin][i];
//    }
//    Protocol_SendMessage(64, ID_ADC_FILTER_VALUES, &f);
//    short i;
////    short j;
////    Protocol_SendMessage(64, ID_ADC_FILTER_VALUES, &Fil[pin]);
////    Protocol_SendMessage(64, ID_ADC_FILTER_VALUES, &A[pin]);
//    int tot =0;
////    int wtot=0;
//    for(i=0;i<FILTERLENGTH;i++){
//         v[i] = A[pin][i];
//    }
//    for(i=0;i<FILTERLENGTH;i++){
////        j=(start-i)%FILTERLENGTH;
//        tot += v[(start-i)%FILTERLENGTH]*(Fil[pin][i]);
////        wtot += Fil[pin][i];
//    }
////    tot =tot/wtot;
////    tot = tot>>15;
//    short FIR=tot&0xFFFF;//divide by 2^15
//    return FIR;

    return ADCFilter_ApplyFilter(Fil[pin],A[pin],start);
}

/**
 * @Function short ADCFilter_ApplyFilter(short filter[], short values[], short startIndex)
 * @param filter, pointer to filter weights
 * @param values, pointer to circular buffer of values
 * @param startIndex, location of first sample so filter can be applied correctly
 * @return Filtered and Scaled Value
 * @brief returns final signal given the input arguments
 * @warning returns a short but internally calculated value should be an int */
short ADCFilter_ApplyFilter(short filter[], short values[], short startIndex){
    short i;
    short j;
//    Protocol_SendMessage(64, ID_ADC_FILTER_VALUES, &filter);
//    Protocol_SendMessage(64, ID_ADC_FILTER_VALUES, &values);
    int tot =0;
    for(i=0;i<FILTERLENGTH;i++){
        j=(startIndex-i)%FILTERLENGTH;
        tot += filter[i]*values[j];
    }
    short FIR=(tot>>15)&0xFFFF;//divide by 2^15
    return FIR;
}

/**
 * @Function ADCFilter_SetWeights(short pin, short weights[])
 * @param pin, which channel to return
 * @param pin, array of shorts to load into the filter for the channel
 * @return SUCCESS or ERROR
 * @brief loads new filter weights for selected channel */
int ADCFilter_SetWeights(short pin, short weights[]){
    short i;
    for(i=0;i<FILTERLENGTH;i++){
        Fil[pin][i] = weights[i];
    }
    return SUCCESS;
}