/**
 * File: FreeRunningTimer.c
 * Author: Dylan Harootunian
 * Created January 19, 2021 10:20PM
 * ECE121 Winter 2021
 */


#include "xc.h"
#include "sys/attribs.h"
#include "stdio.h"
#include "stdlib.h"
#include "BOARD.h"
#include "Protocol.h"
#include "stdint.h"
#include "MessageIDs.h"
#include "GenericTypeDefs.h"
#include "FreeRunningTimer.h"
#include "RotaryEncoder.h"
#include "DCMotorDrive.h"

#define ROLLOVER 0x3FFF;
static unsigned short old =0;
static int rate;
static unsigned int OT =0;
static char testMessage[MAXPAYLOADLENGTH];

// int main(void)
// {
//    BOARD_Init();
//    Protocol_Init();
//    sprintf(testMessage, "DCMotor Compiled at %s %s", __DATE__, __TIME__);
//    Protocol_SendDebugMessage(testMessage);
//    FreeRunningTimer_Init();
//    RotaryEncoder_Init(0x0000);
//    DCMotorDrive_Init();
//    int p;
//    unsigned int t = 100;
//    while(1){
//        if (Protocol_IsMessageAvailable()) {
//            if(Protocol_ReadNextID() == ID_COMMAND_OPEN_MOTOR_SPEED){
//                Protocol_GetPayload(&p);
//                p = Protocol_IntEndednessConversion(p);
//                DCMotorDrive_SetMotorSpeed(p);
//                Protocol_SendMessage(0,ID_COMMAND_OPEN_MOTOR_SPEED_RESP,0x00);
//            }
//        }
//        LEDS_SET(128);
//         if(FreeRunningTimer_GetMilliSeconds()>t){
//            t+=10;
//            unsigned short A = RotaryEncoder_ReadRawAngle();
//            unsigned int T = FreeRunningTimer_GetMilliSeconds();
////            if((T-OT)>=2){ 
////                OT=T;
//            rate=((A-old));//
//            old = A;
//            if((rate>MAXMOTORSPEED) && LATDbits.LATD8){
//                rate = rate - ROLLOVER;
//            }
//            if((rate<-(MAXMOTORSPEED)) && LATDbits.LATD0 ){
//                rate = rate + ROLLOVER;
//            }
////           }
//           rate = rate*50;//scale since its 2 ms
//           rate = rate/(84*8);//get angle of top motor based gear ratio to bottom 
//           rate = Protocol_IntEndednessConversion(rate);
//           Protocol_SendMessage(4, ID_REPORT_RATE,&rate);
//        }
//     }
// }
 
//void  __ISR(_OUTPUT_COMPARE_3_VECTOR) __OC3Interrupt(void) {
//    IFS0bits.OC3IF = 0;
//}
/**
 * @Function DCMotorDrive_Init(void)
 * @param None
 * @return SUCCESS or ERROR
 * @brief initializes timer3 to 2Khz and set up the pins
 * @warning you will need 3 pins to correctly drive the motor  */
int DCMotorDrive_Init(void){
    TRISDbits.TRISD8 = 0;//pins 2 and 3 to outputs
    TRISDbits.TRISD0 = 0;
    LATDbits.LATD8=1;//just pin 2 high
    LATDbits.LATD0=0;
    //timer 3 set up
    T3CON= 0x0;//clearing registers
    T3CONbits.TCKPS = 0b011; //1:8 prescale
    TMR3 = 0x0;
    PR3 = 2500; //every half milisecond
    //output compare 3
    OC3CON= 0;
    OC3CONbits.OCTSEL = 1;
    OC3CONbits.OCM = 0b110;
    OC3R=0;
    OC3RS = 0x000; 
//    T3CONbits.TON = 1;
//    IFS0bits.OC3IF = 0;
//    IPC3bits.OC3IP = 2;
//    IEC0bits.OC3IE = 1;
    //turnign on
    OC3CONbits.ON = 1;
    T3CONbits.ON=1;
}


/**
 * @Function DCMotorDrive_SetMotorSpeed(int newMotorSpeed)
 * @param newMotorSpeed, in units of Duty Cycle (+/- 1000)
 * @return SUCCESS or ERROR
 * @brief Sets the new duty cycle for the motor, 0%->0, 100%->1000 */
int DCMotorDrive_SetMotorSpeed(int newMotorSpeed){
    int temp;
    temp = (PR3*abs(newMotorSpeed))/1000;
    if(newMotorSpeed<0){
        LATDbits.LATD8=1;//just pin 2 high
        LATDbits.LATD0=0;
    }
    else{
        LATDbits.LATD8=0;//just pin 3 high
        LATDbits.LATD0=1;
    }
//    sprintf(testMessage, "motor speed is %d",temp);
//    Protocol_SendDebugMessage(testMessage);
    if(temp>PR3){
        temp=PR3;
    }
    OC3RS = temp;
}


/**
 * @Function DCMotorControl_GetMotorSpeed(void)
 * @param None
 * @return duty cycle of motor 
 * @brief returns speed in units of Duty Cycle (+/- 1000) */
int DCMotorControl_GetMotorSpeed(void){
    int temp = ((OC3RS*1000)/PR3);
    if(LATDbits.LATD8){
       temp = temp*-1;
    }
   return temp;
    
}


/**
 * @Function DCMotorDrive_SetBrake(void)
 * @param None
 * @return SUCCESS or FAILURE
 * @brief set the brake on the motor for faster stop */
int DCMotorDrive_SetBrake(void){
    OC3RS = 0;//stops
}