/**
 * File: FreeRunningTimer.c
 * Author: Dylan Harootunian
 * Created January 19, 2021 10:20PM
 * ECE121 Winter 2021
 */


#include "xc.h"
#include "sys/attribs.h"
#include "stdio.h"
#include "stdlib.h"
#include "BOARD.h"
#include "Protocol.h"
#include "stdint.h"
#include "MessageIDs.h"
#include "GenericTypeDefs.h"
#include "FreeRunningTimer.h"
#include "RotaryEncoder.h"
#include "DCMotorDrive.h"
#include "FeedbackControl.h"

#define MAX_CONTROL_OUTPUT (1<<FEEDBACK_MAXOUTPUT_POWER)

static unsigned char payload[MAXPAYLOADLENGTH];
char message[MAXPAYLOADLENGTH];
static int A;
static int y;
static int Kp;
static int Ki;
static int Kd;

// int main(void)
// {
//     BOARD_Init();
//     Protocol_Init();
//     char testMessage[MAXPAYLOADLENGTH];
//     sprintf(testMessage, "FeedbackControl Compiled at %s %s", __DATE__, __TIME__);
//     Protocol_SendDebugMessage(testMessage);
//     FreeRunningTimer_Init();
//     RotaryEncoder_Init(0x0000);
//     DCMotorDrive_Init();
//     FeedbackControl_Init();
//     unsigned int t = 100;
//     while(1){
//        LEDS_SET(128);
//        if (Protocol_IsMessageAvailable()) {
//            if(Protocol_ReadNextID() == ID_FEEDBACK_SET_GAINS){
//                Protocol_GetPayload(&payload);
//                int i;
//                int P = 0;//gets proportional gains
//                for(i=0;i<4;i++){
//                    P |= payload[i]<<(24-(i*8));
//                }
//                sprintf(testMessage, "first byte is %d ",P);//this reading method pre flips the endedness
//                Protocol_SendDebugMessage(testMessage);
//                int I =0;//gets integral gains
//                for(i=4;i<8;i++){
//                    I |= payload[i]<<(24-((i-4)*8));
//                }
//                sprintf(testMessage, "second byte is %d ",I);
//                Protocol_SendDebugMessage(testMessage);   
//                int D =0;//gets derivative gains
//                for(i=8;i<12;i++){
//                    D |= payload[i]<<(24-((i-8)*8));
//                }
//                sprintf(testMessage, "third byte is %d ",D);
//                Protocol_SendDebugMessage(testMessage);
//                
//                FeedbackControl_SetProportionalGain(P);//set gains
//                FeedbackControl_SetIntegralGain(I);
//                FeedbackControl_SetDerivativeGain(D);  
//                Protocol_SendMessage(0,ID_FEEDBACK_SET_GAINS_RESP,0x00);
//            }
//            if(Protocol_ReadNextID() == ID_FEEDBACK_RESET_CONTROLLER){
//                Protocol_GetPayload(&payload);
//                FeedbackControl_ResetController();
//                sprintf(testMessage, "buffer");
//                Protocol_SendDebugMessage(testMessage);
//                Protocol_SendMessage(0,ID_FEEDBACK_RESET_CONTROLLER_RESP,0x00);
//            }
//            if(Protocol_ReadNextID() == ID_FEEDBACK_UPDATE){
//                Protocol_GetPayload(&payload);
//                int i;
//                int r = 0;//gets proportional gains
//                for(i=0;i<4;i++){
//                    r |= payload[i]<<(24-(i*8));
//                }
//                sprintf(testMessage, "first byte is %d ",r);
//                Protocol_SendDebugMessage(testMessage);
//                int Y =0;//gets integral gains
//                for(i=4;i<8;i++){
//                    Y |= payload[i]<<(24-((i-4)*8));
//                }
//                sprintf(testMessage, "second byte is %d ",Y);
//                Protocol_SendDebugMessage(testMessage);
//                int u = FeedbackControl_Update(r,Y);
//                u = Protocol_IntEndednessConversion(u);
//                Protocol_SendMessage(4,ID_FEEDBACK_UPDATE_OUTPUT,&u);
//            }
//        }
//        if(FreeRunningTimer_GetMilliSeconds()>t){
//           t+=100;
//        }
//     }
// }
int FeedbackControl_Init(void){
    A = 0;
    y = 0;
    Kp = 0;
    Ki = 0;
    Kd = 0;
    return;
}

int FeedbackControl_SetProportionalGain(int newGain){
    Kp = newGain;
    return SUCCESS;
}

/**
 * @Function FeedbackControl_SetIntegralGain(int newGain);
 * @param newGain, integer integral gain
 * @return SUCCESS or ERROR
 * @brief sets the new I gain for controller */
int FeedbackControl_SetIntegralGain(int newGain){
    Ki = newGain;
    return SUCCESS;
}

/**
 * @Function FeedbackControl_SetDerivativeGain(int newGain);
 * @param newGain, integer derivative gain
 * @return SUCCESS or ERROR
 * @brief sets the new D gain for controller */
int FeedbackControl_SetDerivativeGain(int newGain){
    Kd = newGain;
    return SUCCESS;
}

/**
 * @Function FeedbackControl_GetPorportionalGain(void)
 * @param None
 * @return Proportional Gain
 * @brief retrieves requested gain */
int FeedbackControl_GetProportionalGain(void){
    return Kp;
}

/**
 * @Function FeedbackControl_GetIntegralGain(void)
 * @param None
 * @return Integral Gain
 * @brief retrieves requested gain */
int FeedbackControl_GetIntegralGain(void){
    return Ki;
}

/**
 * @Function FeedbackControl_GetDerivativeGain(void)
 * @param None
 * @return Derivative Gain
 * @brief retrieves requested gain */
int FeedbackControl_GetDerivativeGain(void){
    return Kd;
}

/**
 * @Function FeedbackControl_Update(int referenceValue, int sensorValue)
 * @param referenceValue, wanted reference
 * @param sensorValue, current sensor value
 * @brief performs feedback step according to algorithm in lab manual */
int FeedbackControl_Update(int referenceValue, int sensorValue){
    int r = referenceValue;
    int Y = sensorValue;
    int e = r - Y;
    A += e;
    int d = -(Y - y);
    y = Y;
    int u = (Kp*e) + (Ki*A) + (Kd*d);
    if(u>MAX_CONTROL_OUTPUT){
        u =MAX_CONTROL_OUTPUT;
        A -=e;
    }
    if(u<-(MAX_CONTROL_OUTPUT)){
        u = -MAX_CONTROL_OUTPUT;
        A -=e;
    }
//    sprintf(message, "u: %d Error: %d Accumulator: %d", u, Kp * e, Ki * A);
//    Protocol_SendDebugMessage(message);
    return u;
}

/**
 * @Function FeedbackControl_ResetController(void)
 * @param None
 * @return SUCCESS or ERROR
 * @brief resets integrator and last sensor value to zero */
int FeedbackControl_ResetController(void){
    A=0;
    y=0;
    return SUCCESS;
}