/////**
//// * File: FreeRunningTimer.c
//// * Author: Dylan Harootunian
//// * Created January 19, 2021 10:20PM
//// * ECE121 Winter 2021
//// */
////
////
#include "xc.h"
#include "sys/attribs.h"
#include "stdio.h"
#include "stdlib.h"
#include "BOARD.h"
#include "Protocol.h"
#include "stdint.h"
#include "MessageIDs.h"
#include "GenericTypeDefs.h"
#include "FreeRunningTimer.h"
#include "RotaryEncoder.h"
#include "DCMotorDrive.h"
#include "FeedbackControl.h"
#include "NonVolatileMemory.h"
#include "ADCFilter.h"

static unsigned char payload[MAXPAYLOADLENGTH];
static unsigned char gains[MAXPAYLOADLENGTH];
static char testMessage[MAXPAYLOADLENGTH];
short weight[FILTERLENGTH];
#define ROLLOVER 0x3FFF;
#define GEAR 84
#define RATE 8
#define DEGREE 46
#define RANGE_LIMIT 150
#define MS_SCALE 25
static int old =0;
static int rate;
static int totP =0;
static int accP;
static int CA;
//static short fil;
static char M =0;
static short storage[MAXPAYLOADLENGTH];
static int s = 0;
// 5000, 300, 500 tend to be good gains
//4500, 450, 100
//4500, 200, 1000
 int main(void)
 {
    BOARD_Init();
    Protocol_Init();
    sprintf(testMessage, "DCMotor Compiled at %s %s", __DATE__, __TIME__);
    Protocol_SendDebugMessage(testMessage);
    FreeRunningTimer_Init();
    RotaryEncoder_Init(0x0000);
    DCMotorDrive_Init();
    ADCFilter_Init();
    NonVolatileMemory_Init();
    int p;
    unsigned int t = 2;
    unsigned int t2 = 10;
    unsigned int t3 = 1;
    NonVolatileMemory_ReadPage((1)<<6,64, (unsigned char*)gains);
    int i;
    int P = 0;//gets proportional gains
    for(i=0;i<4;i++){
        P |= gains[i]<<(24-(i*8));
    }

    int I =0;//gets integral gains
    for(i=4;i<8;i++){
        I |= gains[i]<<(24-((i-4)*8));
    }

    int D =0;//gets derivative gains
    for(i=8;i<12;i++){
        D |= gains[i]<<(24-((i-8)*8));
    }
    
    FeedbackControl_SetProportionalGain(P);//set gains
    FeedbackControl_SetIntegralGain(I);
    FeedbackControl_SetDerivativeGain(D);
    NonVolatileMemory_ReadPage((2)<<6,64, (unsigned char*)weight);
//    Protocol_SendMessage(64, ID_NVM_READ_PAGE_RESP, &weight);
    ADCFilter_SetWeights(0,weight);
    Protocol_SendMessage(1,ID_LAB5_CUR_MODE,&M);
    Protocol_SendMessage(12,ID_FEEDBACK_CUR_GAINS,&gains);
    CA = totP;
    while(1){
        if (Protocol_IsMessageAvailable()) {
            if(Protocol_ReadNextID() == ID_FEEDBACK_SET_GAINS){
                Protocol_GetPayload(&gains);//sets new gains and saves them to nvm
                int i;
                int P = 0;//gets proportional gains
                for(i=0;i<4;i++){
                    P |= gains[i]<<(24-(i*8));
                }
                int I =0;//gets integral gains
                for(i=4;i<8;i++){
                    I |= gains[i]<<(24-((i-4)*8));
                }     
                int D =0;//gets derivative gains
                for(i=8;i<12;i++){
                    D |= gains[i]<<(24-((i-8)*8));
                }
                FeedbackControl_SetProportionalGain(P);//set gains
                FeedbackControl_SetIntegralGain(I);
                FeedbackControl_SetDerivativeGain(D);  
                Protocol_SendMessage(0,ID_FEEDBACK_SET_GAINS_RESP,0x00);
                NonVolatileMemory_WritePage((1)<<6,64, (unsigned char*)gains);
            }
            if(Protocol_ReadNextID() == ID_FEEDBACK_REQ_GAINS){
                    Protocol_GetPayload(&p);//sets new gains and saves them to nvm
                    NonVolatileMemory_ReadPage((1)<<6,64, (unsigned char*)gains);
                    int i;
                    int P = 0;//gets proportional gains
                    for(i=0;i<4;i++){
                        P |= gains[i]<<(24-(i*8));
                    }

                    int I =0;//gets integral gains
                    for(i=4;i<8;i++){
                        I |= gains[i]<<(24-((i-4)*8));
                    }

                    int D =0;//gets derivative gains
                    for(i=8;i<12;i++){
                        D |= gains[i]<<(24-((i-8)*8));
                    }

                    FeedbackControl_SetProportionalGain(P);//set gains
                    FeedbackControl_SetIntegralGain(I);
                    FeedbackControl_SetDerivativeGain(D);  
                    Protocol_SendMessage(12,ID_FEEDBACK_CUR_GAINS,&gains);
            }
            if(Protocol_ReadNextID() == ID_COMMAND_OPEN_MOTOR_SPEED){
                Protocol_GetPayload(&p);
                p = Protocol_IntEndednessConversion(p);
                DCMotorDrive_SetMotorSpeed(p);
                Protocol_SendMessage(0,ID_COMMAND_OPEN_MOTOR_SPEED_RESP,0x00);
            }
            if(Protocol_ReadNextID() == ID_LAB5_SET_MODE){
                Protocol_GetPayload(&p);
                M = p;
//                Protocol_SendMessage(1,ID_LAB5_CUR_MODE,&M);
            }
            if(Protocol_ReadNextID() == ID_LAB5_REQ_MODE){
                Protocol_GetPayload(&p);
                Protocol_SendMessage(1,ID_LAB5_CUR_MODE,&M);
            }
            if(Protocol_ReadNextID() == ID_COMMANDED_POSITION){
                Protocol_GetPayload(&p);
                CA = p;
                CA = Protocol_IntEndednessConversion(CA);
                if(CA>150){
                    CA= 150;
                }
                if(CA<-150){
                    CA= -150;
                }
                FeedbackControl_ResetController();
            }
            if(Protocol_ReadNextID() == ID_ADC_FILTER_VALUES){
                Protocol_GetPayload(&weight);
                unsigned short i;
                for(i=0; i<FILTERLENGTH; i++){
                    weight[i] = Protocol_ShortEndednessConversion(weight[i]);
                }
                ADCFilter_SetWeights(0,weight);
                NonVolatileMemory_WritePage((2)<<6,64, (unsigned char*)weight);
            }
        }
//        LEDS_SET(128);
         if(FreeRunningTimer_GetMilliSeconds()>t){
            
            t+=2;
            int A = RotaryEncoder_ReadRawAngle();
            rate=((A-old));//
            old = A;
            if((rate>MAXMOTORSPEED) && LATDbits.LATD8){
                rate = rate - ROLLOVER;
                accP = accP - ROLLOVER;
            }
            if((rate<-(MAXMOTORSPEED)) && LATDbits.LATD0 ){
                rate = rate + ROLLOVER;
                accP = accP + ROLLOVER;
            }
           rate = rate*MS_SCALE;//scale since its 2 ms
           rate = rate/(GEAR);//get angle of top motor based gear ratio to bottom 
           rate = Protocol_IntEndednessConversion(rate);
           totP = A + accP;
           totP = totP/(GEAR*DEGREE);
//           totP = Protocol_IntEndednessConversion(totP);
        }
        if(FreeRunningTimer_GetMilliSeconds()>t2){
            t2+=10;
//            LEDS_SET(2);
            if(M){
                int test = ADCFilter_RawReading(0);
//                sprintf(testMessage, "A0 is %d", test);
//                Protocol_SendDebugMessage(testMessage);
                test = test * 300;
                test = test /1023;
                int R = test -150;
//                test = ADCFilter_FilteredReading(0);
                test = 0;
                int i;
                for(i=0; i<MAXPAYLOADLENGTH; i++){
                    test += storage[i];
                }
                test = test/MAXPAYLOADLENGTH;
                test = test * 300;
                test = test / 520;
                int F = test-150;
//                sprintf(testMessage, "A0 filtered is %d", test);
//                Protocol_SendDebugMessage(testMessage);
                CA = R;
            }
            if(CA!=totP){

                int u = FeedbackControl_Update(CA,totP);
//                sprintf(testMessage, "u is %d", u);
//                Protocol_SendDebugMessage(testMessage);
                u = u/100;
                if(totP > RANGE_LIMIT && u >0){
                    LEDS_SET(0xF0);
                    DCMotorDrive_SetBrake();
                }
                else if(totP < -RANGE_LIMIT && u <0){
                    LEDS_SET(0x0F);
                    DCMotorDrive_SetBrake();
                }
                else{
                    DCMotorDrive_SetMotorSpeed(u);
                    LEDS_SET(0x18);
                }
            }
            else{
                DCMotorDrive_SetBrake();
            }
            int data[3];
            data[0]=Protocol_IntEndednessConversion(abs(CA -(totP)));
            data[1]=Protocol_IntEndednessConversion(CA);
            data[2]= Protocol_IntEndednessConversion(totP);
            data[3]=Protocol_IntEndednessConversion(CA);
//            Protocol_SendMessage(4, ID_REPORT_RATE,&rate);
            Protocol_SendMessage(16, ID_LAB5_REPORT,&data);
//            Protocol_SendMessage(4, ID_ENCODER_ABS,&data[2]);
        }
        if(FreeRunningTimer_GetMilliSeconds()>t3){
//            LEDS_SET(3);
            t3+=1;
            storage[s] = ADCFilter_FilteredReading(0);
            s++;
            if (s> MAXPAYLOADLENGTH){
                s=0;
            }
        }
            
     }
 }