/**
 * File: FreeRunningTimer.c
 * Author: Dylan Harootunian
 * Created January 19, 2021 10:20PM
 * ECE121 Winter 2021
 */


#include "xc.h"
#include "sys/attribs.h"
#include "stdio.h"
#include "stdlib.h"
#include "BOARD.h"
#include "Protocol.h"
#include "stdint.h"
#include "MessageIDs.h"
#include "GenericTypeDefs.h"
#include "FrequencyGenerator.h"
#include "NonVolatileMemory.h"
#define EEPROM 0b1010000
#define PAGE 64
static char testMessage[MAXPAYLOADLENGTH];
//int main(void){
//    BOARD_Init();
//    Protocol_Init();
//    sprintf(testMessage, "Protocol Test Compiled at %s %s", __DATE__, __TIME__);
//    Protocol_SendDebugMessage(testMessage);
//    NonVolatileMemory_Init();
//    unsigned char payload [MAXPAYLOADLENGTH];
//    while (1) {
////        LEDS_SET(128);
//        if (Protocol_IsMessageAvailable()) {
//            if (Protocol_ReadNextID() == ID_NVM_WRITE_BYTE) {
//                
//                Protocol_GetPayload(&payload);
////                payload=Protocol_IntEndednessConversion(payload);
////                sprintf(testMessage, "payload is%X",payload);
////                Protocol_SendDebugMessage(testMessage);   
//                unsigned int address = (payload[0]<<24) + (payload [1]<<16)+ (payload [2]<<8) + payload[3];
//                char x = NonVolatileMemory_WriteByte(address, payload[4]);
//                LEDS_SET(1);
//                if (x==ERROR){
//                    sprintf(testMessage, "ERROR in writing byte");
//                    Protocol_SendDebugMessage(testMessage);
//                }
//                if (x==SUCCESS){
//                   Protocol_SendMessage(0, ID_NVM_WRITE_BYTE_ACK, 0x00);
//                }
//            }
//            if (Protocol_ReadNextID() == ID_NVM_READ_BYTE) {    
//                Protocol_GetPayload(&payload);
//                unsigned int address = (payload[0]<<24) + (payload [1]<<16)+ (payload [2]<<8) + payload[3];
//                unsigned char d = NonVolatileMemory_ReadByte(address);
//                LEDS_SET(2);
//                if (d==ERROR){
//                    sprintf(testMessage, "ERROR in reading byte");
//                    Protocol_SendDebugMessage(testMessage);
//                }
//                else{
//                   Protocol_SendMessage(1, ID_NVM_READ_BYTE_RESP, &d);
//                }
//            }
//            if (Protocol_ReadNextID() == ID_NVM_WRITE_PAGE) {
//                
//                Protocol_GetPayload(&payload); 
//                unsigned int address = (payload[0]<<24) + (payload [1]<<16)+ (payload [2]<<8) + payload[3];
//                int y;
//                unsigned char write[MAXPAYLOADLENGTH];
//                for(y=4;y<(MAXPAYLOADLENGTH);y++){
//                    write[y-4]=payload[y];
//
//                }
//                char x = NonVolatileMemory_WritePage((address<<6), PAGE ,write);
//                LEDS_SET(4);
//                if (x==ERROR){
//                    sprintf(testMessage, "ERROR in writing byte");
//                    Protocol_SendDebugMessage(testMessage);
//                }
//                if (x==SUCCESS){
//                   Protocol_SendMessage(0, ID_NVM_WRITE_PAGE_ACK, 0x00);
//                }
//            }
//            if (Protocol_ReadNextID() == ID_NVM_READ_PAGE) {
//                Protocol_GetPayload(&payload);
//                unsigned int address = (payload[0]<<24) + (payload [1]<<16)+ (payload [2]<<8) + payload[3];
//                unsigned char read[MAXPAYLOADLENGTH];
//                unsigned int r = NonVolatileMemory_ReadPage((address<<6), PAGE ,read);
//                LEDS_SET(8);
//                if (r==ERROR){
//                    sprintf(testMessage, "ERROR in reading byte");
//                    Protocol_SendDebugMessage(testMessage);
//                }
//                else{
//                   Protocol_SendMessage(64, ID_NVM_READ_PAGE_RESP, &read);
//                }
//            }
//        }
//    } 
//    return SUCCESS;
// 
//}

/**
 * @Function NonVolatileMemory_Init(void)
 * @param None
 * @return SUCCESS or ERROR
 * @brief initializes I2C for usage */
int NonVolatileMemory_Init(void){
    I2C1CON=0;
    I2C1STAT = 0;
//    I2C1CONbits.SEN = 1;
    
    I2C1BRG = 0x00C5;//sets 40MHZ processor to 100kHZ
    I2C1CONbits.PEN = 1;
    while(I2C1CONbits.PEN ==1){};
//    I2C1CONbits.ACKEN = 1;
    I2C1CONbits.ON = 1;
    return;
}

/**
 * @Function NonVolatileMemory_ReadByte(int address)
 * @param address, device address to read from
 * @return value at said address
 * @brief reads one byte from device
 * @warning Default value for this EEPROM is 0xFF */
unsigned char NonVolatileMemory_ReadByte(int address){
        I2C1CONbits.SEN = 1;//start bit
        while(I2C1CONbits.SEN==1){}//waits for start bit to be sent
        I2C1TRN = EEPROM<<1;//EEPROM Register 
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
        if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
            return ERROR;
        }
        I2C1TRN = (address >>8)&0xFF;// first byte of the address (high)
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
        if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
            return ERROR;
        }
        I2C1TRN = address & 0xFF;// second byte of the address (low)
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
        if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
            return ERROR;
        }
        I2C1CONbits.RSEN = 1;//start bit
        while(I2C1CONbits.RSEN==1){}//waits for start bit to be sent
        I2C1TRN = (EEPROM<<1) + 1;//EEPROM Register 
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
        if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
            return ERROR;
        }
        I2C1CONbits.RCEN = 1;
        while(I2C1CONbits.RCEN){};
        unsigned char data = I2C1RCV;// read data from address 
        I2C1CONbits.ACKDT = 1;//sets NACK to be sent
        I2C1CONbits.ACKEN = 1;//send NACK
        while(I2C1CONbits.ACKEN==1){};
        I2C1CONbits.PEN = 1;//stops
        while(I2C1CONbits.PEN ==1){};//waits for stop bit to be sent
        return data;
}

/**
 * @Function char NonVolatileMemory_WriteByte(int address, unsigned char data)
 * @param address, device address to write to
 * @param data, value to write at said address
 * @return SUCCESS or ERROR
 * @brief writes one byte to device */
char NonVolatileMemory_WriteByte(int address, unsigned char data){
        I2C1CONbits.SEN = 1;//start bit
//        LEDS_SET(data);
        while(I2C1CONbits.SEN==1){}//waits for start bit to be sent
        I2C1TRN = EEPROM<<1;//EEPROM Register 
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
//         I2C1STATbits.R_W = 0;//sets to write mode
         if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
             return ERROR;
         }
         I2C1TRN = (address >>8)&0xFF;// first byte of the address (high)
         while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
         if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
             return ERROR;
         }
         I2C1TRN = address & 0xFF;// second byte of the address (low)
         while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
         if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
             return ERROR;
         }
         I2C1TRN = data;// write data to address
         while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
         if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
             return ERROR;
         }
         I2C1CONbits.PEN = 1;//stops
         while(I2C1CONbits.PEN ==1){};//waits for stop bit to be sent
         
         return SUCCESS;         
}

/**
 * @Function int NonVolatileMemory_ReadPage(int page, char length, unsigned char data[])
 * @param page, page value to read from
 * @param length, value between 1 and 64 bytes to read
 * @param data, array to store values into
 * @return SUCCESS or ERROR
 * @brief reads bytes in page mode, up to 64 at once
 * @warning Default value for this EEPROM is 0xFF */
int NonVolatileMemory_ReadPage(int page, char length, unsigned char data[]){
        I2C1CONbits.SEN = 1;//start bit
        while(I2C1CONbits.SEN==1){}//waits for start bit to be sent
        I2C1TRN = EEPROM<<1;//EEPROM Register 
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
        if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
            return ERROR;
        }
        I2C1TRN = (page >>8)&0xFF;// first byte of the address (high)
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
        if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
            return ERROR;
        }
        I2C1TRN = page & 0xFF;// second byte of the address (low)
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
        if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
            return ERROR;
        }
        I2C1CONbits.RSEN = 1;//start bit
        while(I2C1CONbits.RSEN==1){}//waits for start bit to be sent
        I2C1TRN = (EEPROM<<1) + 1;//EEPROM Register 
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
        if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
            return ERROR;
        }
        int l;
        I2C1CONbits.ACKDT = 0;//sets NACK to be sent       
        for(l=0; l<length;l++){
            while(I2C1CON&0x1F){};//makes sure I2C is stable checks first 5 bits 1F = 00011111
            I2C1CONbits.RCEN = 1;
            while(I2C1CONbits.RCEN){};
            data[l] = I2C1RCV;// read data from address 
//            int i;
//            for (i=0; i < 1500; i++) {
//                asm("nop");
//            }
//            sprintf(testMessage, "data read is %X",data[l]);
//            Protocol_SendDebugMessage(testMessage);
            while(I2C1STATbits.RBF){};
            if(l==length-1){
                I2C1CONbits.ACKDT = 1;//sets ACK to be sent
            }
            I2C1CONbits.ACKEN = 1;//send NACK or ACK
            while(I2C1CONbits.ACKEN==1){};
        }
        
        I2C1CONbits.PEN = 1;//stops
        while(I2C1CONbits.PEN ==1){};//waits for stop bit to be sent
        return data;
}

/**
 * @Function char int NonVolatileMemory_WritePage(int page, char length, unsigned char data[])
 * @param address, device address to write to
 * @param data, value to write at said address
 * @return SUCCESS or ERROR
 * @brief writes one byte to device */
int NonVolatileMemory_WritePage(int page, char length, unsigned char data[]){
        I2C1CONbits.SEN = 1;//start bit
//        LEDS_SET(data);
        while(I2C1CONbits.SEN==1){}//waits for start bit to be sent
        I2C1TRN = EEPROM<<1;//EEPROM Register 
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
//         I2C1STATbits.R_W = 0;//sets to write mode
        if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
            return ERROR;
        }
        I2C1TRN = (page >>8)&0xFF;// first byte of the address (high)
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
        if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
            return ERROR;
        }
        I2C1TRN = page & 0xFF;// second byte of the address (low)
        while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
        if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
            return ERROR;
        }

        int l;  
        for(l=0; l<length;l++){
            I2C1TRN = data[l];// write data to address
            int i;
            for (i=0; i < 5000; i++) {
                asm("nop");
            }
//            sprintf(testMessage, "data wrote is %X",data[l]);
//            Protocol_SendDebugMessage(testMessage);
            while(I2C1STATbits.TRSTAT==1){};//transmit status bit to see that its all transmitted
            if(I2C1STATbits.ACKSTAT ==1){//checks to see if ACK is recevied
                return ERROR;
            }   
        }
        I2C1CONbits.PEN = 1;//stops
        while(I2C1CONbits.PEN ==1){};//waits for stop bit to be sent
         
        return SUCCESS;  
}
