/**
 * File: FreeRunningTimer.c
 * Author: Dylan Harootunian
 * Created march 8, 2021 10:20PM
 * ECE121 Winter 2021
 */


#include "xc.h"
#include "sys/attribs.h"
#include "stdio.h"
#include "stdlib.h"
#include "BOARD.h"
#include "Protocol.h"
#include "stdint.h"
#include "MessageIDs.h"
#include "GenericTypeDefs.h"
#include "FreeRunningTimer.h"
#include "RotaryEncoder.h"

#define MAX_RATE 1000
static char debug[MAXPAYLOADLENGTH];
static unsigned int packet = 0x3FFF;
static unsigned short angle;
static unsigned short old;
static int rate;
static unsigned int OT =0;
// int main(void)
// {
//    BOARD_Init();
//    Protocol_Init();
//    char testMessage[MAXPAYLOADLENGTH];
//    sprintf(testMessage, "Roatary Encoder Compiled at %s %s", __DATE__, __TIME__);
//    Protocol_SendDebugMessage(testMessage);
//    FreeRunningTimer_Init();
//    RotaryEncoder_Init(0x0000);
//
//    unsigned int t = 10;
//    while(1){
//        if(FreeRunningTimer_GetMilliSeconds()>t){
//           unsigned short A = RotaryEncoder_ReadRawAngle();
//           t+=10;
//           LEDS_SET(128);
//           unsigned int T = FreeRunningTimer_GetMilliSeconds();
//           if((T-OT)>=2){
//               LEDS_SET(1);    
//               OT=T;
//               rate=((A-old));//
//               old = A;
//               if(rate>MAX_RATE){
//                   rate = rate - packet;
//               }
//               if(rate<(-MAX_RATE)){
//                   rate = rate + packet;
//               }
//           }
//            A = Protocol_ShortEndednessConversion(A);
//           Protocol_SendMessage(2, ID_ROTARY_ANGLE,&A);
//            rate = Protocol_IntEndednessConversion(rate);
//           Protocol_SendMessage(4, ID_REPORT_RATE,&rate);
//        }
//    }
// }

void  __ISR(_TIMER_2_VECTOR) Timer2IntHandler(void){
//    LEDS_SET(0x01);
    IFS0bits.T2IF = 0;
    LATDbits.LATD3 = 0;//chip select low
    SPI2BUF = packet;
    return;
}

void  __ISR(_SPI_2_VECTOR) __SPI2Interrupt(void) {
//    LEDS_SET(0x80);
    IFS1bits.SPI2RXIF = 0;
    angle = SPI2BUF;
    angle = angle &0x3FFF;//i find dividing by 2^6 gives smoothest output
    LATDbits.LATD3 = 1;
    return;
//    angle = Protocol_ShortEndednessConversion(angle);

}

int RotaryEncoder_Init(char interfaceMode){
    SPI2CON=0;
//    SPI2CON2 = 0;
    //timer 2 set up
    T2CON= 0x0;//clearing registers
    T2CONbits.TCKPS = 0b011; //1:8 prescale
    TMR2 = 0x0;
    //No interupt needed on timer 2
    PR2 = (5000)*2;//5000 is 1 ms 
    IFS0bits.T2IF = 0;//setup interupt
    IPC2bits.T2IP = 5;
    IEC0bits.T2IE = 1;
    SPI2CONbits.MSTEN =1; //sets master mode
    SPI2CONbits.MODE32 = 0; //makes sure its not 32 bit mode
    SPI2CONbits.MODE16 = 1; //sets mode to 16 bit
    SPI2CONbits.SMP = 1; //
    SPI2CONbits.CKP = 0; //
    SPI2CONbits.CKE = 0; //sets mode 1 spi
    SPI2BRG = 3; //sets to 5MHZ
    TRISDbits.TRISD1 = 0; //sets pin 5 to an ouput
    TRISDbits.TRISD3 = 0; //sets pin 9 to an ouput will be chip select pin
    LATDbits.LATD1 = 1; //turns pin on for roatary power
    
    
    IFS1bits.SPI2RXIF = 0;//setup interupt
    IPC7bits.SPI2IP = 6;
    IEC1bits.SPI2RXIE = 1;
    
    SPI2CONbits.ON=1;
    
    LATDbits.LATD3 = 0;//causes interupt to trigger once so that first packet is thrown out
    int i;
    for (i=0; i < 2000; i++) {
        asm("nop");
    }
    unsigned int t = FreeRunningTimer_GetMicroSeconds();
    while(FreeRunningTimer_GetMicroSeconds()<t+1){}//delay 350ns
    int p = Parity_Check(packet);
    packet |= 0x4000;
    packet |= (p<<15);
    SPI2BUF = packet;
//    for (i=0; i < 2000; i++) {
//        asm("nop");
//    }
    
//    LATDbits.LATD3 = 1;
    
    T2CONbits.ON=1;//turn timer on
//    packet = interfaceMode;
    return;
}
unsigned short RotaryEncoder_ReadRawAngle(void){
    return angle;
}
//private function
int Parity_Check(unsigned int in);
int Parity_Check(unsigned int in){
    int p = 0;
    int i;
    for(i=1;i<0x8000;i=i<<1){
        if(in & i){
            p = p+1;
        }
    }
    p = !(p%2);
    return p;
}